import Database from 'better-sqlite3';
import { migrate } from '@blackglory/better-sqlite3-migrations';

import utils from './utils.mjs';
import config from './config.mjs';

// class

export class DB {
    constructor() {
        this.my_db = createDB();
    }
    siteInfo(site) {
        return getSiteInfo(this.my_db, site);
    }
    siteComments(site_url) {
        return getSiteComments(this.my_db, site_url);
    }
    pageComments(site_url, page) {
        return getPageComments(this.my_db, site_url, page);
    }
    insertPageComment(site_url, path, comment) {
        return insertPageComment(this.my_db, site_url, path, comment);
    }
}

const db = new DB();
export default db;

// migrations

function migrations() {
    return [
    { version: 1,
      up: `
CREATE TABLE comment (
  id integer not null,
  site integer not null,
  path text not null,
  user text not null,
  user_website text,
  message text not null,
  published text default (datetime('now')),
  reply_to integer,
  PRIMARY KEY (site, path, id)
);
      `,
      down: `
DROP TABLE comment;
      `
    }
    ];
}

// setup

function createDB() {
    const db = new Database(utils.db_path);
    db.pragma('journal_mode = WAL');
    migrate(db, migrations(), 1);
    return db;
}

// queries

function getSiteInfo(db, site_url) {
    const site = config.getSite(site_url);
    if (!site || !site.info) { throw "Unknown site" }
    return site;
}

function insertPageComment(db, site_url, path, comment) {
    const site = config.getSite(site_url);
    if (!site || !site.info.id) { throw "Unknown site" }

    let object = {...comment, site_id: site.info.id, path: path };

    const stmt = db.prepare(`
        INSERT INTO comment(id, site, path, user, user_website, message, reply_to)
        SELECT
          ( SELECT count(*)
            FROM comment
            WHERE site = @site_id AND path = @path
          ),
          @site_id,
          @path,
          @user,
          @user_website,
          @message,
          @reply_to
        RETURNING
          id as id,
          user,
          user_website,
          message,
          published,
          reply_to
        ;
    `);
    return stmt.all(object);
}

function getPageComments(db, site_url, path) {
    const site = config.getSite(site_url);
    if (!site || !site.info.id) { return []; }
    const stmt = db.prepare(`
        SELECT
          id,
          user,
          user_website,
          message,
          published,
          reply_to
        FROM comment
        WHERE site = @site_id AND path = @path
        ORDER BY id
    ;`);
    return stmt.all({ site_id: site.info.id, path });
}

function getSiteComments(db, site_url) {
    const site = config.getSite(site_url);
    if (!site || !site.info.id) { return []; }
    const stmt = db.prepare(`
        SELECT
          id,
          @site_url as site,
          path,
          user,
          user_website,
          message,
          published
        FROM
          comment
        WHERE site = @site_id
        ORDER BY published DESC
        ;
    `);
    return stmt.all({ site_id: site.info.id, site_url });
}
